@@ -1,8 +1,16 @@
-use aries_askar::{Backend, Entry, EntryTag, ErrorKind, Store, TagFilter};
+use std::{fmt::Debug, future::Future, ops::Deref, pin::Pin, sync::Arc};
+
+use aries_askar::{
+    kms::{KeyAlg, LocalKey},
+    Backend, Entry, EntryTag, Error, ErrorKind, Store, TagFilter,
+};
+
+use tokio::task::spawn;
 
 const ERR_PROFILE: &'static str = "Error creating profile";
 const ERR_SESSION: &'static str = "Error starting session";
 const ERR_TRANSACTION: &'static str = "Error starting transaction";
+const ERR_COMMIT: &'static str = "Error committing transaction";
 const ERR_COUNT: &'static str = "Error performing count";
 const ERR_FETCH: &'static str = "Error fetching test row";
 const ERR_FETCH_ALL: &'static str = "Error fetching all test rows";
@@ -13,12 +21,27 @@ const ERR_REPLACE: &'static str = "Error replacing test row";
 const ERR_REMOVE_ALL: &'static str = "Error removing test rows";
 const ERR_SCAN: &'static str = "Error starting scan";
 const ERR_SCAN_NEXT: &'static str = "Error fetching scan rows";
-// const ERR_CREATE_KEYPAIR: &'static str = "Error creating keypair";
-// const ERR_FETCH_KEY: &'static str = "Error fetching key";
-// const ERR_SIGN: &'static str = "Error signing message";
-// const ERR_VERIFY: &'static str = "Error verifying signature";
+const ERR_CREATE_KEYPAIR: &'static str = "Error creating keypair";
+const ERR_INSERT_KEY: &'static str = "Error inserting key";
+const ERR_FETCH_KEY: &'static str = "Error fetching key";
+const ERR_LOAD_KEY: &'static str = "Error loading key";
 
-pub async fn db_create_remove_profile<DB: Backend>(db: &Store<DB>) {
+pub trait TestStore: Clone + Deref<Target = Store<Self::DB>> + Send + Sync {
+    type DB: Backend + Debug + 'static;
+
+    fn close(self) -> Pin<Box<dyn Future<Output = Result<(), Error>>>>;
+}
+
+impl<B: Backend + Debug + 'static> TestStore for Arc<Store<B>> {
+    type DB = B;
+
+    fn close(self) -> Pin<Box<dyn Future<Output = Result<(), Error>>>> {
+        let db = Arc::try_unwrap(self).unwrap();
+        Box::pin(db.close())
+    }
+}
+
+pub async fn db_create_remove_profile(db: impl TestStore) {
     let profile = db.create_profile(None).await.expect(ERR_PROFILE);
     assert_eq!(
         db.remove_profile(profile)
@@ -34,13 +57,13 @@ pub async fn db_create_remove_profile<DB: Backend>(db: &Store<DB>) {
     );
 }
 
-pub async fn db_fetch_fail<DB: Backend>(db: &Store<DB>) {
+pub async fn db_fetch_fail(db: impl TestStore) {
     let mut conn = db.session(None).await.expect(ERR_SESSION);
     let result = conn.fetch("cat", "name", false).await.expect(ERR_FETCH);
     assert_eq!(result.is_none(), true);
 }
 
-pub async fn db_insert_fetch<DB: Backend>(db: &Store<DB>) {
+pub async fn db_insert_fetch(db: impl TestStore) {
     let test_row = Entry::new(
         "category",
         "name",
@@ -78,7 +101,7 @@ pub async fn db_insert_fetch<DB: Backend>(db: &Store<DB>) {
     assert_eq!(rows[0], test_row);
 }
 
-pub async fn db_insert_duplicate<DB: Backend>(db: &Store<DB>) {
+pub async fn db_insert_duplicate(db: impl TestStore) {
     let test_row = Entry::new("category", "name", "value", Vec::new());
 
     let mut conn = db.session(None).await.expect(ERR_SESSION);
@@ -106,7 +129,7 @@ pub async fn db_insert_duplicate<DB: Backend>(db: &Store<DB>) {
     assert_eq!(err.kind(), ErrorKind::Duplicate);
 }
 
-pub async fn db_insert_remove<DB: Backend>(db: &Store<DB>) {
+pub async fn db_insert_remove(db: impl TestStore) {
     let test_row = Entry::new("category", "name", "value", Vec::new());
 
     let mut conn = db.session(None).await.expect(ERR_SESSION);
@@ -126,14 +149,14 @@ pub async fn db_insert_remove<DB: Backend>(db: &Store<DB>) {
         .expect(ERR_REQ_ROW);
 }
 
-pub async fn db_remove_missing<DB: Backend>(db: &Store<DB>) {
+pub async fn db_remove_missing(db: impl TestStore) {
     let mut conn = db.session(None).await.expect(ERR_SESSION);
 
     let err = conn.remove("cat", "name").await.expect_err(ERR_REQ_ERR);
     assert_eq!(err.kind(), ErrorKind::NotFound);
 }
 
-pub async fn db_replace_fetch<DB: Backend>(db: &Store<DB>) {
+pub async fn db_replace_fetch(db: impl TestStore) {
     let test_row = Entry::new("category", "name", "value", Vec::new());
 
     let mut conn = db.session(None).await.expect(ERR_SESSION);
@@ -168,7 +191,7 @@ pub async fn db_replace_fetch<DB: Backend>(db: &Store<DB>) {
     assert_eq!(row, replace_row);
 }
 
-pub async fn db_replace_missing<DB: Backend>(db: &Store<DB>) {
+pub async fn db_replace_missing(db: impl TestStore) {
     let test_row = Entry::new("category", "name", "value", Vec::new());
 
     let mut conn = db.session(None).await.expect(ERR_SESSION);
@@ -186,7 +209,7 @@ pub async fn db_replace_missing<DB: Backend>(db: &Store<DB>) {
     assert_eq!(err.kind(), ErrorKind::NotFound);
 }
 
-pub async fn db_count<DB: Backend>(db: &Store<DB>) {
+pub async fn db_count(db: impl TestStore) {
     let category = "category".to_string();
     let test_rows = vec![Entry::new(&category, "name", "value", Vec::new())];
 
@@ -213,7 +236,7 @@ pub async fn db_count<DB: Backend>(db: &Store<DB>) {
     assert_eq!(count, 0);
 }
 
-pub async fn db_count_exist<DB: Backend>(db: &Store<DB>) {
+pub async fn db_count_exist(db: impl TestStore) {
     let test_row = Entry::new(
         "category",
         "name",
@@ -352,7 +375,7 @@ pub async fn db_count_exist<DB: Backend>(db: &Store<DB>) {
     );
 }
 
-pub async fn db_scan<DB: Backend>(db: &Store<DB>) {
+pub async fn db_scan(db: impl TestStore) {
     let category = "category".to_string();
     let test_rows = vec![Entry::new(
         &category,
@@ -400,7 +423,7 @@ pub async fn db_scan<DB: Backend>(db: &Store<DB>) {
     assert_eq!(rows, None);
 }
 
-pub async fn db_remove_all<DB: Backend>(db: &Store<DB>) {
+pub async fn db_remove_all(db: impl TestStore) {
     let test_rows = vec![
         Entry::new(
             "category",
@@ -460,117 +483,30 @@ pub async fn db_remove_all<DB: Backend>(db: &Store<DB>) {
     assert_eq!(removed, 2);
 }
 
-// pub async fn db_keypair_create_fetch<DB: Backend>(db: &Store<DB>) {
-//     let mut conn = db.session(None).await.expect(ERR_SESSION);
+pub async fn db_keypair_insert_fetch(db: impl TestStore) {
+    let keypair = LocalKey::generate(KeyAlg::Ed25519, false).expect(ERR_CREATE_KEYPAIR);
 
-//     let metadata = "meta".to_owned();
-//     let key_info = conn
-//         .create_keypair(KeyAlg::Ed25519, Some(&metadata), None, None)
-//         .await
-//         .expect(ERR_CREATE_KEYPAIR);
-//     assert_eq!(key_info.params.metadata, Some(metadata));
+    let mut conn = db.session(None).await.expect(ERR_SESSION);
 
-//     let found = conn
-//         .fetch_key(key_info.category.clone(), &key_info.ident, false)
-//         .await
-//         .expect(ERR_FETCH_KEY);
-//     assert_eq!(Some(key_info), found);
-// }
+    let key_name = "testkey";
+    let metadata = "meta";
+    conn.insert_key(&key_name, &keypair, Some(metadata), None, None)
+        .await
+        .expect(ERR_INSERT_KEY);
 
-// pub async fn db_keypair_sign_verify<DB: Backend>(db: &Store<DB>) {
-//     let mut conn = db.session(None).await.expect(ERR_SESSION);
+    let found = conn
+        .fetch_key(&key_name, false)
+        .await
+        .expect(ERR_FETCH_KEY)
+        .expect(ERR_REQ_ROW);
+    assert_eq!(found.algorithm(), Some(KeyAlg::Ed25519.as_str()));
+    assert_eq!(found.name(), key_name);
+    assert_eq!(found.metadata(), Some(metadata));
+    assert_eq!(found.is_local(), true);
+    found.load_local_key().expect(ERR_LOAD_KEY);
+}
 
-//     let key_info = conn
-//         .create_keypair(KeyAlg::Ed25519, None, None, None)
-//         .await
-//         .expect(ERR_CREATE_KEYPAIR);
-
-//     let message = b"message".to_vec();
-//     let sig = conn
-//         .sign_message(&key_info.ident, &message)
-//         .await
-//         .expect(ERR_SIGN);
-
-//     assert_eq!(
-//         verify_signature(&key_info.ident, &message, &sig).expect(ERR_VERIFY),
-//         true
-//     );
-
-//     assert_eq!(
-//         verify_signature(&key_info.ident, b"bad input", &sig).expect(ERR_VERIFY),
-//         false
-//     );
-
-//     assert_eq!(
-//         verify_signature(
-//             &key_info.ident,
-//             // [0u8; 64]
-//             b"xt19s1sp2UZCGhy9rNyb1FtxdKiDGZZPNFnc1KiM9jYYEuHxuwNeFf1oQKsn8zv6yvYBGhXa83288eF4MqN1oDq",
-//             &sig
-//         ).expect(ERR_VERIFY),
-//         false
-//     );
-
-//     assert_eq!(
-//         verify_signature(&key_info.ident, &message, b"bad sig").is_err(),
-//         true
-//     );
-
-//     let err = verify_signature("not a key", &message, &sig).expect_err(ERR_REQ_ERR);
-//     assert_eq!(err.kind(), ErrorKind::Input);
-// }
-
-// pub async fn db_keypair_pack_unpack_anon<DB: Backend>(db: &Store<DB>) {
-//     let mut conn = db.session(None).await.expect(ERR_SESSION);
-
-//     let recip_key = conn
-//         .create_keypair(KeyAlg::Ed25519, None, None, None)
-//         .await
-//         .expect(ERR_CREATE_KEYPAIR);
-
-//     let msg = b"message".to_vec();
-
-//     let packed = conn
-//         .pack_message(vec![recip_key.ident.as_str()], None, &msg)
-//         .await
-//         .expect(ERR_PACK);
-
-//     let (unpacked, p_recip, p_send) = conn.unpack_message(&packed).await.expect(ERR_UNPACK);
-//     assert_eq!(unpacked, msg);
-//     assert_eq!(p_recip.to_string(), recip_key.ident);
-//     assert_eq!(p_send, None);
-// }
-
-// pub async fn db_keypair_pack_unpack_auth<DB: Backend>(db: &Store<DB>) {
-//     let mut conn = db.session(None).await.expect(ERR_SESSION);
-
-//     let sender_key = conn
-//         .create_keypair(KeyAlg::Ed25519, None, None, None)
-//         .await
-//         .expect(ERR_CREATE_KEYPAIR);
-//     let recip_key = conn
-//         .create_keypair(KeyAlg::Ed25519, None, None, None)
-//         .await
-//         .expect(ERR_CREATE_KEYPAIR);
-
-//     let msg = b"message".to_vec();
-
-//     let packed = conn
-//         .pack_message(
-//             vec![recip_key.ident.as_str()],
-//             Some(&sender_key.ident),
-//             &msg,
-//         )
-//         .await
-//         .expect(ERR_PACK);
-
-//     let (unpacked, p_recip, p_send) = conn.unpack_message(&packed).await.expect(ERR_UNPACK);
-//     assert_eq!(unpacked, msg);
-//     assert_eq!(p_recip.to_string(), recip_key.ident);
-//     assert_eq!(p_send.map(|k| k.to_string()), Some(sender_key.ident));
-// }
-
-pub async fn db_txn_rollback<DB: Backend>(db: &Store<DB>) {
+pub async fn db_txn_rollback(db: impl TestStore) {
     let test_row = Entry::new("category", "name", "value", Vec::new());
 
     let mut conn = db.transaction(None).await.expect(ERR_TRANSACTION);
@@ -598,7 +534,7 @@ pub async fn db_txn_rollback<DB: Backend>(db: &Store<DB>) {
     assert_eq!(row, None);
 }
 
-pub async fn db_txn_drop<DB: Backend>(db: &Store<DB>) {
+pub async fn db_txn_drop(db: impl TestStore) {
     let test_row = Entry::new("category", "name", "value", Vec::new());
 
     let mut conn = db
@@ -628,7 +564,7 @@ pub async fn db_txn_drop<DB: Backend>(db: &Store<DB>) {
 }
 
 // test that session does NOT have transaction rollback behaviour
-pub async fn db_session_drop<DB: Backend>(db: &Store<DB>) {
+pub async fn db_session_drop(db: impl TestStore) {
     let test_row = Entry::new("category", "name", "value", Vec::new());
 
     let mut conn = db.session(None).await.expect(ERR_SESSION);
@@ -654,7 +590,7 @@ pub async fn db_session_drop<DB: Backend>(db: &Store<DB>) {
     assert_eq!(row, Some(test_row));
 }
 
-pub async fn db_txn_commit<DB: Backend>(db: &Store<DB>) {
+pub async fn db_txn_commit(db: impl TestStore) {
     let test_row = Entry::new("category", "name", "value", Vec::new());
 
     let mut conn = db.transaction(None).await.expect(ERR_TRANSACTION);
@@ -669,7 +605,7 @@ pub async fn db_txn_commit<DB: Backend>(db: &Store<DB>) {
     .await
     .expect(ERR_INSERT);
 
-    conn.commit().await.expect("Error committing transaction");
+    conn.commit().await.expect(ERR_COMMIT);
 
     let mut conn = db.session(None).await.expect(ERR_SESSION);
 
@@ -680,7 +616,7 @@ pub async fn db_txn_commit<DB: Backend>(db: &Store<DB>) {
     assert_eq!(row, Some(test_row));
 }
 
-pub async fn db_txn_fetch_for_update<DB: Backend>(db: &Store<DB>) {
+pub async fn db_txn_fetch_for_update(db: impl TestStore) {
     let test_row = Entry::new("category", "name", "value", Vec::new());
 
     let mut conn = db.transaction(None).await.expect(ERR_TRANSACTION);
@@ -711,5 +647,90 @@ pub async fn db_txn_fetch_for_update<DB: Backend>(db: &Store<DB>) {
     assert_eq!(rows.len(), 1);
     assert_eq!(rows[0], test_row);
 
-    conn.commit().await.expect("Error committing transaction");
+    conn.commit().await.expect(ERR_COMMIT);
+}
+
+pub async fn db_txn_contention(db: impl TestStore + 'static) {
+    let test_row = Entry::new(
+        "category",
+        "count",
+        "0",
+        vec![
+            EntryTag::Encrypted("t1".to_string(), "v1".to_string()),
+            EntryTag::Plaintext("t2".to_string(), "v2".to_string()),
+        ],
+    );
+
+    let mut conn = db.transaction(None).await.expect(ERR_TRANSACTION);
+
+    conn.insert(
+        &test_row.category,
+        &test_row.name,
+        &test_row.value,
+        Some(test_row.tags.as_slice()),
+        None,
+    )
+    .await
+    .expect(ERR_INSERT);
+
+    conn.commit().await.expect(ERR_COMMIT);
+
+    const TASKS: usize = 10;
+    const INC: usize = 1000;
+
+    async fn inc(db: impl TestStore, category: String, name: String) -> Result<(), &'static str> {
+        // try to avoid panics in this section, as they will be raised on a tokio worker thread
+        for _ in 0..INC {
+            let mut conn = db.transaction(None).await.expect(ERR_TRANSACTION);
+            let row = conn
+                .fetch(&category, &name, true)
+                .await
+                .map_err(|e| {
+                    log::error!("{:?}", e);
+                    ERR_FETCH
+                })?
+                .ok_or(ERR_REQ_ROW)?;
+            let val: usize = str::parse(row.value.as_opt_str().ok_or("Non-string counter value")?)
+                .map_err(|_| "Error parsing counter value")?;
+            conn.replace(
+                &category,
+                &name,
+                &format!("{}", val + 1).as_bytes(),
+                Some(row.tags.as_slice()),
+                None,
+            )
+            .await
+            .map_err(|e| {
+                log::error!("{:?}", e);
+                ERR_REPLACE
+            })?;
+            conn.commit().await.map_err(|_| ERR_COMMIT)?;
+        }
+        Ok(())
+    }
+
+    let mut tasks = vec![];
+    for _ in 0..TASKS {
+        tasks.push(spawn(inc(
+            db.clone(),
+            test_row.category.clone(),
+            test_row.name.clone(),
+        )));
+    }
+
+    // JoinSet is not stable yet, just await all the tasks
+    for task in tasks {
+        if let Err(s) = task.await.unwrap() {
+            panic!("Error in concurrent update task: {}", s);
+        }
+    }
+
+    // check the total
+    let mut conn = db.session(None).await.expect(ERR_SESSION);
+    let row = conn
+        .fetch(&test_row.category, &test_row.name, false)
+        .await
+        .expect(ERR_FETCH)
+        .expect(ERR_REQ_ROW);
+    assert_eq!(row.value, format!("{}", TASKS * INC).as_bytes());
 }
